/*
 *    RemoveAttributesFilter.java
 *    Copyright (C) 2010 Poznan University of Technology, Poznan, Poland
 *    @author Dariusz Brzezinski (dariusz.brzezinski@cs.put.poznan.pl)
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package moa.streams.filters;

import java.util.Random;

import moa.core.AutoExpandVector;
import moa.core.DoubleVector;
import moa.core.GaussianEstimator;
import moa.core.InstancesHeader;
import moa.options.FloatOption;
import moa.options.IntOption;
import moa.options.StringOption;
import weka.core.Attribute;
import weka.core.FastVector;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.Range;

public class RemoveAttributesFilter extends AbstractStreamFilter {

	@Override
	public String getPurposeString() {
		return "Removes selected attributes from the stream.";
	}
	
	private static final long serialVersionUID = 1L;

	/**
	 * Attributes to remove.
	 */
	public StringOption attributesOption = new StringOption("attributes", 'a',
			"Attributes that will be removed from the stream.", "-1");

	private Range selectionRange = new Range();
	private int [] selectedAttributes;
	private InstancesHeader outputFormat;
	
	protected Random random;

	protected AutoExpandVector<Object> attValObservers;

	/**
	 * Creates a new filtered header of the data stream.
	 */
	public InstancesHeader getHeader() {		
		this.selectionRange.setRanges(this.attributesOption.getValue());	
		this.selectionRange.setUpper(this.inputStream.getHeader().numAttributes()-1);

	    // Create the output buffer
	    FastVector attributes = new FastVector();
	    int outputClass = -1;
	    this.selectionRange.setInvert(true);
	    this.selectedAttributes = this. selectionRange.getSelection();
	    
	    for (int i = 0; i < this.selectedAttributes.length; i++) {
	      int current = this.selectedAttributes[i];
	      if (this.inputStream.getHeader().classIndex() == current) {
	    	  outputClass = attributes.size();
	      }
	      Attribute keep = (Attribute)this.inputStream.getHeader().attribute(current).copy();
	      attributes.addElement(keep);
	    }

	    this.outputFormat = new InstancesHeader(new Instances(this.inputStream.getHeader().relationName(), attributes, 0));
	    this.outputFormat.setClassIndex(outputClass);
		
		return this.outputFormat;
	}

	/**
	 * Returns the next filtered example.
	 */
	public Instance nextInstance() {
		if(this.attributesOption.getValue() == "-1")
		{
			return this.inputStream.nextInstance();
		}
		
		selectionRange.setInvert(false);
	    selectedAttributes = selectionRange.getSelection();
		Instance inst = this.inputStream.nextInstance();
		inst.setDataset(null);
		
		for (int i = 0; i < selectedAttributes.length; i++) {
			inst.deleteAttributeAt(selectedAttributes[i]);
		}
		
		inst.setDataset(this.outputFormat);
		
		return inst;
	}

	/**
	 * Not implemented.
	 */
	public void getDescription(StringBuilder sb, int indent) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void restartImpl() {
	}

}
